@@ -1,6 +1,6 @@
 use utf8;
 package CatalystX::InjectModule::MI;
-$CatalystX::InjectModule::MI::VERSION = '0.14';
+$CatalystX::InjectModule::MI::VERSION = '0.15';
 # This plugin is inspired by :
 # - CatalystX::InjectComponent
 # - Catalyst::Plugin::AutoCRUD
@@ -21,6 +21,7 @@ use YAML qw(Dump DumpFile LoadFile);
 use Term::ANSIColor qw(:constants);
 use Path::Tiny;
 use Path::Class qw( file );
+use ExtUtils::Installed;
 
 has debug => (
               is       => 'rw',
@@ -66,6 +67,12 @@ has _static_dirs => (
               default  => sub { [] },
           );
 
+has libs => (
+             is       => 'rw',
+             isa      => 'ArrayRef',
+             default  => sub { [] },
+            );
+
 
 sub log {
     my($self, $msg, $level) = @_;
@@ -92,6 +99,7 @@ sub resolv {
     my $operation = shift;
     my $version   = shift;
 
+    next if ! $module;
     my $Module   = $self->get_module($module, $operation, $version );
     die "Module $module not found !" if ! defined $Module->{name};
 
@@ -112,15 +120,25 @@ sub load {
 
     $self->resolver(Dependency::Resolver->new(debug => $self->debug ));
 
+    # By default search also in INC
+    my $search_localy_only = 0;
+
     # search modules in 'path' directories
     for my $dir ( @{ $conf->{path} } ) {
-        if ( $dir eq '__INC__' ) {
-            pop(@INC) if $INC[-1] eq '.'; # do not search module in '.'
-            push(@{$conf->{path}}, @INC);
+        if ( $dir eq '__NO_INC__' ) {
+            $search_localy_only = 1;
             next;
         }
         $self->_load_modules_path($dir, $conf_filename);
     }
+    # Search modules in @INC
+    if ( ! $search_localy_only ){
+        for my $dir ( @INC ) {
+            next if ( $dir eq '.');
+            $self->_load_modules_path($dir, $conf_filename, 1);
+        }
+    }
+    
     # Merge config resolved modules ----------------
     $self->_merge_resolved_configs;
 
@@ -167,6 +185,7 @@ sub _add_to_modules_loaded {
     my $all = {};
     foreach my $m ( @$modules ) {
         next if ( $all->{$m->{name}} );
+        
         push(@{$self->modules_loaded},$m);
         $all->{$m->{name}} = 1;
     }
@@ -184,14 +203,14 @@ sub _load_modules_path{
     my $self           = shift;
     my $dir            = shift;
     my $conf_filename  = shift;
-
+    my $from_inc       = shift;
+    
     $self->log("  - search modules in $dir ...");
 
     my $all_configs = $self->_search_in_path( $dir, "^$conf_filename\$" );
-
-    CONFIG: for my $config ( @$all_configs ) {
+  CONFIG: for my $config ( @$all_configs ) {
         my $cfg = Config::Any->load_files({files => [$config], use_ext => 1 })
-            or die "Error (conf: $config) : $!\n";
+          or die "Error (conf: $config) : $!\n";
 
         my($filename, $mod_config) = %{$cfg->[0]};
 
@@ -200,15 +219,19 @@ sub _load_modules_path{
 
         # next if module already added ( ex: path=share + share/modules)
         for my $m ( @{$self->resolver->modules->{$mod_config->{name}}} ) {
-            if ( $path eq $m->{path}){
+            if ( $path eq $m->{path}) {
                 next CONFIG;
-            };
+            }
+            ;
         }
 
         my $msg = "    - find module ". $mod_config->{name};
         $msg .= " v". $mod_config->{version} if defined $mod_config->{version};
+        $msg .= " from INC"  if $from_inc;
         $self->log($msg);
 
+        # for local module (cxim_config is in share/)
+        $path =~ s|/share$||;
         $mod_config->{path} = $path;
 
         $self->resolver->add($mod_config);
@@ -233,8 +256,6 @@ sub _inject {
     # Inject static ----------------------
     $self->_load_static($module);
 
-    # install_module is used when all modules are loaded
-    #$self->install_module($module);
 }
 
 
@@ -257,42 +278,68 @@ sub _merge_resolved_configs {
 }
 
 sub _build_local_config_file{
-  my $self  = shift;
+    my $self  = shift;
 
-  my $file = lc($self->ctx . "::local.yml");
-  $file    =~ s/::/_/g;
-  my $conf        = $self->ctx->config;
-  my $conf_path   = file( File::Spec->rel2abs($file) );
-  my $config_file = path($conf_path->relative);
-
-  # Generates the local configuration file if it doesnot exist
-  $config_file->spew_utf8( Dump($conf) )
-    if ! -e $file;
+    my $file = lc($self->ctx . "::local.yml");
+    $file    =~ s/::/_/g;
+    my $conf        = $self->ctx->config;
+    my $conf_path   = file( File::Spec->rel2abs($file) );
+    my $config_file = path($conf_path->relative);
 
+    # Generates the local configuration file if it doesnot exist
+    if ( ! -e $file ) {
+        $config_file->spew_utf8( Dump($conf) )
+    } else {
+    # Merge loaded conf with local conf
+        my $newconf = LoadFile($config_file);
+        $self->ctx->config( Catalyst::Utils::merge_hashes( $newconf, $self->ctx->config ) );
+        $config_file->spew_utf8( Dump(Catalyst::Utils::merge_hashes( $newconf, $self->ctx->config) ));
+    }
 }
-
 sub _load_lib {
-  my ( $self, $module ) = @_;
+    my ( $self, $module ) = @_;
 
-  my $libpath = $module->{path} . '/lib';
-  return if ( ! -d $libpath);
+    my $all_libs = [];
 
-  $self->log(BLUE."  - Add lib $libpath".CLEAR);
-  unshift( @INC, $libpath );
+    # use module localy
+    if ( -d $module->{path} . '/lib') {
+        $module->{libpath} = $module->{path} . '/lib';
+        $all_libs = $self->_search_in_path( $module->{libpath}, '.pm$' );
+    }
+    # or from installed packages
+    elsif ( -d $module->{path} ){
+        my $module_file = $module->{name};
+        $module_file =~ s|::|/|g;
+        $module_file .= ".pm";
+        my ($inst) = ExtUtils::Installed->new( skip_cwd => 1 );
+        my @all_module_files = $inst->files($module->{name});
+        foreach my $f ( @all_module_files ) {
+            push(@$all_libs,$f) if ( $f =~ /\.pm$/ ); 
 
-  # Search and load components
-  my $all_libs = $self->_search_in_path( $module->{path}, '.pm$' );
+            if ( $f =~ s/$module_file$// ) {
+                $module->{libpath} = $f;
+            }
+            # Nothing
+        }
+    }
+    else {
+        return;
+    }
 
-  foreach my $file (@$all_libs) {
+    $self->log(BLUE."  - Add lib "  . $module->{libpath} . CLEAR);
+    unshift( @INC, $module->{libpath} );
 
-    next if grep {/TraitFor/} $file;
+    $self->libs(\@INC);
 
-    $self->_load_component( $module, $file )
-      if ( grep {/Model|View|Controller/} $file );
+    foreach my $file (@$all_libs) {
 
-    push(@{$self->_view_files}, $file)
-      if ( grep {/\/View\/\w*\.pm/} $file );
-  }
+        next if grep {/TraitFor/} $file;
+        $self->_load_component( $module, $file )
+          if ( grep {/Model|View|Controller/} $file );
+
+        push(@{$self->_view_files}, $file)
+          if ( grep {/\/View\/\w*\.pm/} $file );
+    }
 }
 
 sub install_module {
@@ -300,16 +347,17 @@ sub install_module {
     my $module = shift;
 
     my $module_name = $module->{name};
-    $module_name =~ s|::|/|;
+    my $module_path = $module_name;
+
+    $module_path =~ s|::|/|g;
 
     if ( $self->_is_installed($module) ) {
         $self->log("  - $module_name already installed", 2);
         return;
     }
 
-    my $module_path = $module->{path};
-    my $module_file = $module_path . '/lib/' . $module_name . '.pm';
-
+    my $module_libpath = $module->{libpath};
+    my $module_file    = $module_libpath . '/' . $module_path . '.pm';
     if ( -f $module_file ) {
         load_class($module_name);
         my $mod = $module_name->new( mi => $self);
@@ -333,8 +381,8 @@ sub uninstall_module {
         return;
     }
 
-    my $module_path = $module->{path};
-    my $module_file = $module_path . '/lib/' . $module_name . '.pm';
+    my $module_libpath = $module->{libpath};
+    my $module_file = $module_libpath . '/' . $module_name . '.pm';
 
     if ( -f $module_file ) {
         load_class($module_name);
@@ -458,7 +506,7 @@ sub _load_catalyst_plugin {
 sub _load_template {
 	my ( $self, $module ) = @_;
 
-    foreach my $dir ( 'root/src', 'root/lib') {
+    foreach my $dir ( 'share/root/src', 'share/root/lib', 'root/src', 'root/lib') {
 
         my $template_dir = $module->{path} . "/$dir";
 
@@ -476,40 +524,41 @@ sub _load_template {
 
 sub _load_static {
     my ( $self, $module ) = @_;
-
+    
     my $static_dir = $module->{path} . "/root/static";
 
+    foreach my $static_dir ( $module->{path} . "/share/root/static", $module->{path} . "/root/static" ) {
 
-    if ( -d $static_dir ) {
-        $self->log("  - Add static directory");
-        $module->{static_dir} = $static_dir;
-        push(@{$self->_static_dirs}, $static_dir);
+        if ( -d $static_dir ) {
+            $self->log("  - Add static directory");
+            $module->{static_dir} = $static_dir;
+            push(@{$self->_static_dirs}, $static_dir);
+        }
     }
 }
 
 sub _load_component {
-	my ( $self, $module, $file ) = @_;
-
-	my $libpath = $module->{path} . '/lib';
-	my $comp    = $file;
-	$comp =~ s|$libpath/||;
-	$comp =~ s|\.pm$||;
-	$comp =~ s|/|::|g;
-
-    my $into = $self->ctx;
-    my $as  = $comp;
-    $as =~ s/.*(Model|View|Controller):://;
-	$self->log("  - Add Component into: $into comp:$comp as:$as");
-
-    Catalyst::Utils::inject_component( into => $into,
-                                       component => $comp,
-                                       as => $as );
-
-}
+        my ( $self, $module, $file ) = @_;
+         
+        my $libpath = $module->{libpath};
+        my $comp    = $file;
+        $comp =~ s|$libpath/?||;
+            $comp =~ s|\.pm$||;
+        $comp =~ s|/|::|g;
+         
+        my $into = $self->ctx;
+        my $as  = $comp;
+        $as =~ s/.*(Model|View|Controller):://;
+        $self->log("  - Add Component into: $into comp:$comp as:$as");
+         
+        Catalyst::Utils::inject_component( into => $into,
+                                               component => $comp,
+                                               as => $as );
+    }
 
 sub _search_in_path {
 	my $self  = shift;
-    my $path  = shift;
+        my $path  = shift;
 	my $regex = shift;
 
 	my @files;
@@ -518,8 +567,8 @@ sub _search_in_path {
 		return if !/$regex/;
 
 		my $file = $File::Find::name;
-		push @files, $file;
-	};
+                push @files, $file;
+            };
 
     find( $tf_finder, $path  );
 	return \@files;
@@ -532,7 +581,7 @@ CatalystX::InjectModule::MI Catalyst Module injector
 
 =head1 VERSION
 
-version 0.14
+version 0.15
 
 =head1 SYNOPSIS
 
